package exParser;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.InputStreamReader;

/**
 * Created by IntelliJ IDEA.
 * User: AkiraSasaki
 * Date: 2008/06/01
 * Time: 18:12:38
 * To change this template use File | Settings | File Templates.
 */

enum CharClass{
    WS, LETTER, DIGIT, OTHERS, DELIMITER
}

public class ExTokenizer {
    StringBuilder stringBuffer;
    CharReader reader;

    public ExTokenizer(CharReader reader) {
        this.reader = reader;
        stringBuffer = new StringBuilder();
    }

    public TokenClass nextToken() {
        char ch;
        stringBuffer.setLength(0);
        int nextState = 1;
        do {    // 󔒁Asǂݔ΂B
            ch = reader.nextChar();
        } while (charClass(ch)==CharClass.WS);
        while (true) {
            switch (nextState) {
                case 1: {
                    if (ch == 0)
                        return TokenClass.EOF;
                    if (charClass(ch) == CharClass.LETTER){
                        stringBuffer.append(ch);
                        nextState = 2;
                        break;
                    }
                    stringBuffer.append(ch);
                    if (ch == '(')
                        return TokenClass.LPAR;
                    else if (ch == ')')
                        return TokenClass.RPAR;
                    else if (ch == '+')
                        return TokenClass.PLUS;
                    else if (ch == '*')
                        return TokenClass.MULT;
                    else
                        throw new Error("scanner: illegal character: " + ch);
                }
                case 2: {
                    ch = reader.nextChar();
                    if (charClass(ch) == CharClass.LETTER ||
                            charClass(ch) == CharClass.DIGIT) {
                        stringBuffer.append(ch);
                        nextState = 2;
                    }
                    else
                        nextState = 5;
                    break;
                }
                case 5:
                    reader.backChar();
                    return TokenClass.IDENT;
                default: {
                    throw new RuntimeException("should not reach");
                }
            }
        }
    }

    public String currentString(){
        return stringBuffer.toString();
    }

    private CharClass charClass(char c){
        if (c >='a' && c <= 'z' || c>='A' && c<= 'Z')
            return CharClass.LETTER;
        if (c >='0' && c <= '9')
            return CharClass.DIGIT;
        if ( c == ' ' || c == '\n')
            return CharClass.WS;
        else if (c == ';')
            return CharClass.DELIMITER;
        else {
            return CharClass.OTHERS;
        }
    }

    public static void main(String [] args) throws Exception {
        BufferedReader reader = null;
        if (args.length == 1) {
            File input = new File(args[0]);
            if (!input.canRead()) {
                System.out.println("Error: could not read ["+input+"]");
            }
            reader = new BufferedReader (new FileReader(input));
        }
        else {
            reader = new BufferedReader (new InputStreamReader(System.in));
        }
        // eXgF̐擪g[NǌށBacceptȂΏI
        ExTokenizer s = new ExTokenizer(new CharReader(reader));
        while (s.nextToken()!=null)
            System.out.println(s.currentString());
        reader.close();
    }
}
